library('DESeq2')
library('limma')
# bioconductor stuff first
library('cowplot')
library('data.table')
library('fitdistrplus')
library('readr')

theme_set(theme_light() +
            theme(axis.text = element_text(color = 'black'),
                  strip.text = element_text(color = 'black'),
                  panel.grid.minor = element_blank(),
                  legend.margin = margin(t = 0, r = 0, b = 0, l = 0, unit = 'cm')))

resultDir = 'results'

########################################

colData = setDT(read_csv('params/PRJNA297287_metadata_ena.csv'))

seqSummary1 = colData[, .(read_count = sum(read_count)),
                      by = .(secondary_sample_accession, sample_title)]
seqSummary2 = seqSummary1[startsWith(sample_title, 'WT_AL'),
                          .(mean_read_count = mean(read_count))]
# mean reads per sample: 68 million

colData = unique(colData[, .(sample = secondary_sample_accession, title = sample_title)])

colDataSra = setDT(read_tsv('params/PRJNA297287_SraRunTable.txt'))
colDataSra = unique(colDataSra[, .(sample = SRA_Sample, feeding, genotype)])

colData = merge(colData, colDataSra, by = 'sample')
colData = colData[feeding == 'Ad Libitum' & genotype == 'Wild type']

colData[, time := as.numeric(substring(title, 9, 10))]
colData[, cosine := cos(time * 2 * pi / 24)]
colData[, sine := sin(time * 2 * pi / 24)]

########################################

exprData = readRDS('processed/PRJNA297287_tximport_output.rds')
minCounts = 10

exprData$counts = exprData$counts[, colData$sample]
idx = rowSums( exprData$counts >= minCounts) >= (ncol(exprData$counts) / 2)
exprData$counts = exprData$counts[idx, ]

dds = DESeqDataSetFromTximport(exprData, colData, ~ cosine + sine)
dds = estimateSizeFactors(dds)
dds = estimateDispersionsGeneEst(dds)

dds = estimateDispersionsFit(dds, fitType = 'parametric')
dispFuncParametric = dispersionFunction(dds)

dds = estimateDispersionsFit(dds, fitType = 'local')
dds = estimateDispersionsMAP(dds, outlierSD = 3)

dispFuncLocal = dispersionFunction(dds)

saveRDS(list(local = dispFuncLocal, parametric = dispFuncParametric),
        file.path(resultDir, 'mouse_disp_funcs.rds'))

########################################

dmixnormnow = function(x, mean1, mean2, sd1, sd2, pro) {
  if (pro < 0 | pro > 1 | sd1 <= 0 | sd2 <= 0) {
    return(rep(NA, length(x)))}
  KScorrect::dmixnorm(x, c(mean1, mean2), c(sd1, sd2), c(pro, 1 - pro))}

pmixnormnow = function(x, mean1, mean2, sd1, sd2, pro) {
  if (pro < 0 | pro > 1 | sd1 <= 0 | sd2 <= 0) {
    return(rep(NA, length(x)))}
  KScorrect::pmixnorm(x, c(mean1, mean2), c(sd1, sd2), c(pro, 1 - pro))}

qmixnormnow = function(x, mean1, mean2, sd1, sd2, pro) {
  if (pro < 0 | pro > 1 | sd1 <= 0 | sd2 <= 0) {
    return(rep(NA, length(x)))}
  KScorrect::qmixnorm(x, c(mean1, mean2), c(sd1, sd2), c(pro, 1 - pro))}

log2Means = log2(mcols(dds)$baseMean)
log2MeanFitNorm = fitdist(log2Means, 'norm')
log2MeanFitMix = fitdist(log2Means, 'mixnormnow',
                         start = list(mean1 = 5, mean2 = 10,
                                      sd1 = 1, sd2 = 2, pro = 0.2))

# mixture is literally infinitely better than single gaussian,
# but sampling from it is too complicated for the simphony examples.
# fitting it is also fragile wrt minCounts and start vector.
exp((log2MeanFitNorm$bic - log2MeanFitMix$bic) / 2)

saveRDS(list(norm = log2MeanFitNorm, mix = log2MeanFitMix),
        file.path(resultDir, 'mouse_log2_mean_fits.rds'))

########################################

# library('mgcv')
# means = mcols(dds)$baseMean
# disps = mcols(dds)$dispGeneEst
# logMeans = log2(means)
# logDisps = log2(disps)
# gamFit = gam(logDisps ~ s(logMeans), subset = log(disps) >= -10)
#
# plot(log2(means), log2(disps), pch = '.', ylim = c(-10, 3))
# points(log2(means), log2(dispFuncParametric(means)), pch = '.', col = 'orange')
# points(log2(means), log2(dispFuncLocal(means)), pch = '.', col = 'blue')
# points(log2(means), predict(gamFit, data.frame(logMeans = logMeans)),
#        pch = '.', col = 'green')
#
# rmse = function(r) sqrt(mean(r^2))
# rmseParam = rmse(log(dispFuncParametric(means)) - log(disps))
# rmseLocal = rmse(log(dispFuncLocal(means)) - log(disps))
# rmseGam = rmse(log(2^(predict(gamFit, data.frame(logMeans = logMeans)))) - log(disps))

########################################

dt = data.table(baseMean = mcols(dds)$baseMean,
                disp = mcols(dds)$dispGeneEst)
dt = dt[log(disp) >= -10]

dt[, parametric := log(dispFuncParametric(baseMean)) - log(disp)]
dt[, local := log(dispFuncLocal(baseMean)) - log(disp)]
dtMelt = melt(dt, measure.vars = c('parametric', 'local'),
              variable.name = 'fitType', value.name = 'resid')

dtMelt[, .(rmse = sqrt(mean(resid ^ 2))), by = fitType]

pFit = ggplot(dt) +
  geom_point(aes(x = baseMean, y = disp), size = 0.1, alpha = 0.1) +
  stat_function(aes(color = 'parametric'), fun = dispFuncParametric, size = 0.7) +
  stat_function(aes(color = 'local'), fun = dispFuncLocal, size = 0.7) +
  labs(x = 'Mean normalized counts', y = 'Dispersion') +
  scale_x_continuous(limits = 10 ^ c(0.5, 6), breaks = 10 ^ (0:6), trans = 'log10') +
  #, labels = scales::comma
  scale_y_continuous(limits = c(5e-4, 10), trans = 'log10',
                     labels = function(x) base::format(x, scientific = FALSE)) +
  scale_color_brewer(palette = 'Set2', name = 'Fit type')

# pResid = ggplot(dtMelt) +
#   facet_wrap(~ fitType, nrow = 1) +
#   geom_point(aes(x = baseMean, y = resid), size = 0.1, alpha = 0.25) +
#   stat_smooth(aes(x = baseMean, y = resid)) +
#   labs(x = 'Mean normalized counts', y = 'Residual log dispersion') +
#   scale_x_continuous(trans = 'log10', breaks = 10^(1:6), labels = scales::comma)

########################################

set.seed(6022)
nGenes = 1e4

sampledBaseMeans = 2 ^ unlist(quantile(log2MeanFitNorm, runif(nGenes))$quantiles)
sampledDt = data.table(baseMean = sampledBaseMeans)

sdLogDisp = sqrt(attributes(dispFuncLocal)$dispPriorVar)

sampledDt[, dispResid := rnorm(nGenes, sd = sdLogDisp)]
sampledDt[, disp := dispFuncLocal(baseMean) * exp(dispResid)]

pSamp = ggplot(sampledDt) +
  geom_point(aes(x = baseMean, y = disp), size = 0.1, alpha = 0.1) +
  stat_function(fun = dispFuncLocal, color = '#66c2a5', size = 0.7) +
  labs(x = 'Mean normalized counts', y = 'Dispersion') +
  scale_x_continuous(limits = 10 ^ c(0.5, 6),
                     breaks = 10 ^ (0:6), trans = 'log10') +
  #, labels = scales::comma
  scale_y_continuous(limits = c(5e-4, 10), trans = 'log10',
                     labels = function(x) base::format(x, scientific = FALSE))

p = plot_grid(pFit, pSamp, ncol = 1, labels = 'AUTO',
              align = 'hv', axis = 'tblr')

ggsave(file.path(resultDir, 'mouse_fit_sampled.pdf'),
       plot = p, width = 5, height = 5.5)
ggsave(file.path(resultDir, 'mouse_fit_sampled.png'),
       plot = p, width = 5, height = 5.5)

########################################

# y = edgeR::DGEList(exprData$counts)
# y = edgeR::calcNormFactors(y)
# v = voom(y, design = model.matrix(~ cosine + sine, data = colData))
# fit = lmFit(v)
# fit = eBayes(fit)
#
# rhyLimma = topTable(fit, coef = 2:3, number = Inf, sort.by = 'none')
# rhyLimma = data.table(rhyLimma, keep.rownames = TRUE)
# rhyLimma[, scaleFactor := apply(log2(exprData$counts + 0.5), 1, sd) * sqrt(2)]
# rhyLimma[, amp := sqrt(cosine^2 + sine^2)]
# rhyLimma[, ampScaled := amp / scaleFactor]
# rhyLimma[, ampSimph := ampScaled / sqrt(0.5 * (1 - ampScaled^2))]
#
# # hist(rhyLimma[, ampScaled], breaks = 30)
# # hist(rhyLimma[adj.P.Val <= 0.01, ampSimph], breaks = 30, xlim = c(0, 4.5))
# # hist(log2(rhyLimma[adj.P.Val <= 0.01, ampSimph]), breaks = 30)
# # plot(colData$time, log2(exprData$counts['ENSMUSG00000032311', ] + 1))
#
# log2AmpFit = fitdist(log2(rhyLimma[adj.P.Val <= 0.01, ampSimph]), 'norm')
# saveRDS(log2AmpFit, file.path(resultDir, 'mouse_log2_amp_fit.rds'))
