source(file.path('scripts', 'sim_setup.R'))

interval = 0.1
nReps = 100
period = 24
periodNorm = period / 2 / pi

########################################
# non-rhythmic gaussian

featureGroups = CJ(base = c(-5, 0, 5), sd = c(1, 2, 4))

simData = simphony(featureGroups, nFeatures = nrow(featureGroups),
                   interval = interval, nReps = nReps,
                   family = 'gaussian')

d = mergeSimData(simData, simData$featureMetadata$feature)
nonrhyGauss = d[, .(Family = 'Gaussian',
                    meanExp = base0, meanObs = mean(abund),
                    sdExp = sd, sdObs = stats::sd(abund)),
                by = .(base0, sd)]
nonrhyGauss[, c('base0', 'sd') := NULL]

########################################
# non-rhythmic negative binomial

featureGroups = CJ(base = 4:10)

simData = simphony(featureGroups, nFeatures = nrow(featureGroups),
                   interval = interval, nReps = nReps,
                   family = 'negbinom')

d = mergeSimData(simData, simData$featureMetadata$feature)
nonrhyNegbinom = d[, .(Family = 'Negative binomial',
                       meanExp = 2 ^ base0, meanObs = mean(abund),
                       sdExp = sqrt(2 ^ base0 + (2 ^ base0) ^ 2 * dispFunc[[1]](2 ^ base0)),
                       sdObs = stats::sd(abund)),
                   by = base0]
nonrhyNegbinom[, base0 := NULL]

########################################
# rhythmic gaussian

featureGroups = CJ(amp = c(1, 2, 3), phase = c(0, 6, 12))

simData = simphony(featureGroups, nFeatures = nrow(featureGroups),
                   interval = interval, nReps = nReps,
                   family = 'gaussian')

sampleMetadata = simData$sampleMetadata
sampleMetadata[, time_cos := cos(time / periodNorm)]
sampleMetadata[, time_sin := sin(time / periodNorm)]
design = model.matrix(~ time_cos + time_sin, data = sampleMetadata)

abundData = simData$abundData
fit = lmFit(abundData, design)

d = data.table(coef(fit)[, -1], keep.rownames = TRUE)
setnames(d, 'rn', 'feature')
d = merge(simData$featureMetadata, d, by = 'feature')

# pull out the trig identities
d[, ampObs := sqrt(time_cos ^ 2 + time_sin ^ 2)]
d[, phaseObs := ((pi / 2 - atan2(time_sin, time_cos)) * periodNorm) %% period]

rhyGauss = d[, .(Family = 'Gaussian',
                 ampExp = amp0, ampObs,
                 phaseExp = phase, phaseObs)]

########################################
# rhythmic negative binomial

featureGroups = CJ(amp = c(1, 2, 3), phase = c(0, 6, 12), base = 16)

simData = simphony(featureGroups, nFeatures = nrow(featureGroups),
                   interval = interval, nReps = nReps,
                   family = 'negbinom')

sampleMetadata = simData$sampleMetadata
sampleMetadata[, time_cos := cos(time / periodNorm)]
sampleMetadata[, time_sin := sin(time / periodNorm)]
design = model.matrix(~ time_cos + time_sin, data = sampleMetadata)

abundData = log2(simData$abundData)
fit = lmFit(abundData, design)

d = data.table(coef(fit)[, -1], keep.rownames = TRUE)
setnames(d, 'rn', 'feature')
d = merge(simData$featureMetadata, d, by = 'feature')

# pull out the trig identities
d[, ampObs := sqrt(time_cos ^ 2 + time_sin ^ 2)]
d[, phaseObs := ((pi / 2 - atan2(time_sin, time_cos)) * periodNorm) %% period]

rhyNegbinom = d[, .(Family = 'Negative binomial',
                    ampExp = amp0, ampObs,
                    phaseExp = phase, phaseObs)]

########################################

nonrhy = rbind(nonrhyGauss, nonrhyNegbinom)
setnames(nonrhy, c('meanExp', 'meanObs', 'sdExp', 'sdObs'),
         c('Expected mean', 'Observed mean', 'Expected std dev', 'Observed std dev'))

fwrite(nonrhy, file.path(resultDir, 'sim_validation_nonrhy.tsv'), sep = '\t')

rhy = rbind(rhyGauss, rhyNegbinom)
setnames(rhy, c('ampExp', 'ampObs', 'phaseExp', 'phaseObs'),
         c('Expected amp', 'Observed amp', 'Expected phase', 'Observed phase'))

fwrite(rhy, file.path(resultDir, 'sim_validation_rhy.tsv'), sep = '\t')
